from sentence_transformers import SentenceTransformer
import torch
from torch import F


class MiniLM:
    def __init__(self, model="all-MiniLM-L6-v2"):
        self.model = SentenceTransformer(model)

    def sort_by_centroid(self, sentences: list[str]) -> list[str]:
        embeddings = torch.tensor(self.model.encode(sentences))
        normalized_vectors = F.normalize(embeddings, p=2, dim=1)
        centroid = normalized_vectors.mean(dim=0, keepdim=True)
        similarities = torch.matmul(normalized_vectors, centroid.T).flatten()
        sorted_indices = torch.argsort(similarities, descending=True)
        return [sentences[i] for i in sorted_indices]


from transformers import Blip2Processor, Blip2ForConditionalGeneration


class Blip2:
    def __init__(self, model="Salesforce/blip2-opt-2.7b"):
        self.processor = Blip2Processor.from_pretrained(model)
        self.model = Blip2ForConditionalGeneration(model)

    def describe(self, image) -> str:
        inputs = self.processor(images=image, return_tensors="pt")
        outputs = self.model.generate(**inputs)
        description = self.processor.tokenizer.decode(outputs[0], skip_special_tokens=True)
        return description


from setuptools import setup, find_packages

setup(
    name="yudkow-models",  # Уникальное имя вашего пакета
    version="0.1.3",         # Версия пакета
    author="BlTT",       # Автор пакета
    author_email="bltt6956@gmail.com",  # Email автора
    description="My package with models",  # Краткое описание
    packages=find_packages(),  # Автоматический поиск пакетов
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",  # Лицензия (например, MIT)
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",  # Минимальная версия Python
    install_requires=[],  # Список зависимостей (оставьте пустым, если их нет)
)


from transformers import AutoTokenizer, AutoModelForSeq2SeqLM
import torch


class T5:
    def __init__(self, model="google/flan-t5-base"):
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.tokenizer = AutoTokenizer.from_pretrained(model)
        self.model = AutoModelForSeq2SeqLM.from_pretrained(model).to(self.device)

    def request(self, text: str) -> str:
        input_ids = self.tokenizer(text, return_tensors="pt").input_ids.to(self.device)
        outputs = self.model.generate(input_ids, max_new_tokens=30).to(self.device)
        return self.tokenizer.decode(outputs[0], skip_special_tokens=True)

    def keywords_to_text(self, text: str) -> str:
        return self.request("Generate a sentence with the following keywords: " + text)


from openai import OpenAI

class ChatGPT:
    def __init__(self, api_key):
        self.client = OpenAI(api_key=api_key)

    def ask(self, request: str):
        completion = self.client.chat.completions.create(
            model="gpt-4o-mini",
            messages=[
                {"role": "user",
                 "content": request}
                ])
        return str(completion.choices[0].message)

    def paraphrase(self, text: str):
        template = "Paraphrase the following sentence: \"{}\"\nRespond only with the paraphrased sentence, without any additional comments."
        return self.ask(template.format(text))
